
rm(list = ls())
setwd('/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/')

source("modules/Loading/Loading.R")
source("modules/Calibration/Calibration.R")
source("modules/Anomalies/Anomalies.R")
source("modules/Skill/Skill.R")
source("modules/Saving/Saving.R")
source("modules/Visualization/Visualization.R")
source("tools/prepare_outputs.R")

## Load recipe
#recipe_file <- "recipes/recipe_testing_nadia.yml"
recipe_file <- "recipes/recipe_decathlon.yml"

recipe <- prepare_outputs(recipe_file)

## Load datasets
data <- load_datasets(recipe)

## Calibrate datasets
calibrated_data <- calibrate_datasets(recipe, data)

## Compute anomalies
anomalies <- compute_anomalies(recipe, data)
 
## Calculate skill metrics
skill_metrics <- compute_skill_metrics(recipe, calibrated_data)

# Compute percentiles and probability bins
#probabilities <- compute_probabilities(recipe, calibrated_data$hcst)

# Export all data to netCDF
##save_data(recipe, skill_metrics, skill_metrics, probabilities) ## ?? error
save_data(recipe, data, skill_metrics = skill_metrics) 

# Plot data
plot_data(recipe, data, skill_metrics, significance = T)

# # Plot only the skill metrics:
# plot_data(recipe, archive, data, skill_metrics = skill_metrics, significance = F)
# 
# plot_data(recipe, anomalies, skill_metrics = skill_metrics['crpss'], significance = T)
# 
# # Or only the forecast ensemble mean:
# plot_data(recipe, archive, data, calibrated_data = calibrated_data)
# 
# plot_data(recipe, archive, data, calibrated_data, skill_metrics, significance = T)
# 


##### Check data #####
dim(data$obs$data)
dim(data$hcst$data)
dim(data$fcst$data)


## Define latitude and longitude
lat <- as.vector(data$obs$lat)
lon <- as.vector(data$obs$lon)


## Define plotting parameters
library(RColorBrewer)
startdate <- '01-05'
leadtime <- '0'
years <- '1991-2020'
plot_palette_clim <- rev(brewer.pal(11,'RdBu'))
plot_palette_met <- rev(brewer.pal(10,'PuOr'))



########################### CALCULATE CLIMATOLOGIES ############################


# Convert Kelvins to degrees Celsius
data$obs$data_celcius <- data$obs$data - 273.15
data$hcst$data_celcius <- data$hcst$data -273.15


## Observational climatology
obs_clim <- multiApply::Apply(data = data$obs$data, target_dims = c('ensemble', 'syear'),
                              fun = mean, na.rm = FALSE, ncores = 1)[[1]]

## Model climatology
exp_clim <- multiApply::Apply(data = data$hcst$data, target_dims = c('ensemble', 'syear'),
                              fun = mean, na.rm = FALSE, ncores = 1)[[1]]

## Plot both climatologies 
# PlotLayout(PlotEquiMap, c('lat', 'lon'), var = c(obs_clim, exp_clim), nrow = 1, ncol = 2,
#            lon = lon, lat = lat, filled.continents = FALSE, title_scale = 0.5,
#            titles = c("ERA5 climatology", "ECMWF System5 climatology"), units = 'ºC',
#            cols = rev(brewer.pal(11,'RdBu')), brks = seq(-2, 31, 3),
#            width = 10, height = 6)
# 

## Plot obs clim data
s2dv::PlotEquiMap(obs_clim[1,1,1,1,1,,], lon = lon, lat = lat, filled.cont = F,
                  toptitle = "ERA5 climatology", title_scale = 1, units = "ºC",
                  cols = plot_palette_clim, # brks = seq(-2, 31, 3),
                  fileout = "./figures/climatology_obs_tas_1993-2016_s01.png"
                  )

## Plot exp clim data
s2dv::PlotEquiMap(exp_clim[1,1,1,1,1,,], lon = lon, lat = lat, filled.cont = F,
                  toptitle = "ECMWF System5 climatology", title_scale = 1, units = "ºC",
                  cols = plot_palette_clim, brks = seq(-2, 31, 3),
                  )

################################################################################
########################### RAW DATA METRIC PLOTS ##############################
################################################################################


######### MEAN BIAS of raw data ##########

mean_bias <- (multiApply::Apply(data = data$hcst$data, target_dims = c('ensemble', 'syear'), 
                                fun = mean, na.rm = FALSE, ncores = 1)[[1]]
              - multiApply::Apply(data = data$obs$data, target_dims = c('ensemble', 'syear'),
                                  fun = mean, na.rm = FALSE, ncores = 1)[[1]])

s2dv::PlotEquiMap(var = drop(mean_bias), lon = lon, lat = lat, filled.cont = F,
                  toptitle = paste0 ("Mean bias (sdate: ",startdate, ", leadtime: ", leadtime, ", years: ", years, ") - VS"),
                  title_scale = 0.5, units = "ºC", cols = plot_palette_met, brks = seq(-2.5,2.5,0.5), triangle_ends = c(T,T), col_inf = '#2d004b', col_sup = '#7f3b08')

####### CORRELATION of raw data ########

## Calculate 95% confidence interval (EnsCorr only)
enscorr_sign <- skill_metrics$enscorr_p.value[[1]] >= 0.05  

## Plot Enscorr
s2dv::PlotEquiMap(var = drop(skill_metrics$enscorr[[1]]), lon = lon, lat = lat,  dots = drop(enscorr_sign), dot_size = 2, filled.continents = FALSE, 
                  toptitle = paste0 ("EnsCorr (sdate: ",startdate, ", leadtime: ", leadtime, ", years: ", years,") - VS"),
                  title_scale = 0.5, cols = plot_palette_met, brks = seq(-1,1,0.2), triangle_ends = c(F,F))

## Plot EnsCorr_specs (does not include significance)
s2dv::PlotEquiMap(var = drop(skill_metrics$enscorr$enscorr_specs), lon = lon, lat = lat,  filled.continents = FALSE, 
                  toptitle = paste0 ("EnsCorr_specs (sdate: ",startdate, ", leadtime: ", leadtime, ", years: ", years,") - VS"),
                  title_scale = 0.5, cols = plot_palette_met, brks = seq(-1,1,0.2), triangle_ends = c(F,F))


###### RPSS of raw data ########

## Plot RPSS
s2dv::PlotEquiMap(var = drop(skill_metrics$rpss[[1]]), lon = lon, lat = lat, dots = drop(!skill_metrics$rpss_significance[[1]]), dot_size = 2, filled.continents = FALSE, 
                  toptitle = paste0("RPSS (sdate: ",startdate, ", leadtime: ", leadtime, ", years: ", years,") - VS"),
                  title_scale = 0.5, cols = plot_palette_met, brks = seq(-1,1,0.2), triangle_ends = c(T,F), col_inf = '#2d004b')




################################################################################
########################## CALIBRATED DATA PLOTS ###############################
################################################################################

## Define calibration method
title_metric_cal = 'EVMOS' #bias, EVMOS, MSE min, CPRS min



######## MEAN BIAS of calibrated data #########

## Calculate ensemble mean of calibrated model 
cal_ensemble_mean <- multiApply::Apply(data = calibrated_data$hcst$data, target_dims = c('ensemble', 'syear'), fun = mean, na.rm = FALSE, ncores = 1)[[1]]

cal_mean_bias <- cal_ensemble_mean - multiApply::Apply(data = data$obs$data, target_dims = c('ensemble', 'syear'), fun = mean, na.rm = FALSE, ncores = 1)[[1]]

s2dv::PlotEquiMap(var = cal_mean_bias, lon = lon, lat = lat, filled.cont = F,
                  toptitle = paste0("Mean bias - ", title_metric_cal, " calibrated", " (sdate: ",startdate, ", leadtime: ", leadtime, ", years: ", years, ") -VS"),
                  title_scale = 0.4, units = "ºC", cols = plot_palette_met, brks = seq(-2.5,2.5,0.5))



####### CORRELATION of calibrated data ########

## Calculate 95% confidence interval (EnsCorr only)
cal_enscorr_sign <- cal_skill_metrics$enscorr_p.value[[1]] >= 0.05  

## Plot calibrated EnsCorr
s2dv::PlotEquiMap(var = drop(cal_skill_metrics$enscorr[[1]]), lon = lon, lat = lat,  dots = drop(cal_enscorr_sign), dot_size = 2, filled.continents = FALSE, 
                  toptitle = paste0 ("EnsCorr - ", title_metric_cal, " calibrated", " (sdate: ",startdate, ", leadtime: ", leadtime, ", years: ", years,") - VS"),
                  title_scale = 0.5, cols = plot_palette_met, brks = seq(-1,1,0.2), triangle_ends = c(F,F))



######## RPSS of calibrated data #########

## Plot calibrated RPSS 
s2dv::PlotEquiMap(var = drop(cal_skill_metrics$rpss[[1]]), lon = lon, lat = lat, dots = drop(!cal_skill_metrics$rpss_significance[[1]]), dot_size = 2, filled.continents = FALSE,
                  toptitle = paste0 ("RPSS - ", title_metric_cal, " calibrated", " (sdate: ",startdate, ", leadtime: ", leadtime, ", years: ", years,") - VS"), title_scale = 0.5,
                  cols = plot_palette_met, brks = seq(-1,1,0.2), triangle_ends = c(T,F), col_inf = '#2d004b')

