#'Function to create all single system/reference scorecards
#' 
#'@description Scorecards function to create scorecard tables for one system and 
#'  one reference combination (types 1 to 4).
#'@param input_data is an array of spatially aggregated metrics containing the
#'   following dimensions; system, reference, metric, region, sdate, ftime.
#'@param system a vector of character strings defining the systems following the
#'   archive.yml format from verification suite 
#'@param reference a vector of character strings defining the references 
#'  following the archive.yml format from verification suite 
#'@param var a character string following the format from 
#'  variable-dictionary.yml from verification suite 
#'@param start.year a numeric indicating the start year of the reference period
#'@param end.year a numeric indicating the end year of the reference period
#'@param start.date a vector of character strings indicating the start months
#'@param forecast.month a vector of numeric indicating the forecast months
#'@param region.names a vector of character strings containing names of the 
#'  regions corresponding to the input data
#'@param metrics a vector of character strings containing the metrics,  
#'  NOTE: must be in same order as in data.
#'@param fileout a path of the location to save the scorecard plots
#'
#'@return
#' This function returns 4 scorecards images, saved in the directory 'fileout'

#'@example
#' scorecard <- SC_create_table_simple(data = aggregated_metrics,
#'                                     system.name = c('ecmwfs5','dwds21'),
#'                                     reference.name = 'era5',
#'                                     var = 'tas', 
#'                                     start.year = 1993,
#'                                     end.year = 2016,
#'                                     start.months = 1:12,
#'                                     forecast.months = 1:6,
#'                                     region.names = c('Tropics', 'Extra-tropical NH', 'Extra-tropical SH')
#'                                     metrics = c('mean_bias', 'enscorr', 'rpss','crpss', 'enssprerr'),
#'                                     fileout = '/esarchive/scratch/nmilders/scorecards_data/saved_results'
#'                                     )
 
  
SC_create_table_simple <- function(data,
                                    system,
                                    reference,
                                    var,
                                    start.year,
                                    end.year,
                                    start.months,
                                    forecast.months,
                                    region.names,
                                    metrics,
                                    cross.validation,
                                    metric.aggregation,
                                    fileout){
  
  ## For tesing
  # data <- aggregated_metrics
  # system <- c('ecmwfs5')
  # reference <-'era5'
  # var <- 'tas'
  # start.year <- 1993
  # end.year <- 2016
  # start.months <- sprintf("%02d", 1:12)
  # forecast.months <- 1:6
  # region.names <- c('Tropics', 'Extra-tropical NH', 'Extra-tropical SH')
  # metrics <-  c('mean_bias', 'enscorr', 'rpss', 'rpss_score_aggr', 'enssprerr')
  #metrics <- c('mean_bias', 'enscorr', 'rpss','crpss', 'enssprerr')
  # fileout <- '/esarchive/scratch/nmilders/scorecards_data/saved_results'
 
  ## Load functions
  source("./modules/Scorecards/SC_transform_data.R")
  source("./modules/Scorecards/SC_plot_simple_by_region.R")
  source("./modules/Scorecards/SC_plot_simple_by_forecast_month.R")

  ## Checks to apply:
  # first dimension in aggregated_metrics is system and second dimension is reference
  # to allow 1 region - if region = 1 --> only scorecards 1 & 3 need to be plotted

  ## Make sure input_data is in correct order for using in functions:
  data_order <- c('system','reference','metric','time','sdate','region')
  data <- Reorder(data, data_order)
  
  ## Identify metrics loaded
  metrics.loaded <-  attributes(data)$metrics

  ## Select only the metrics to visualize from data
  input_data <- Subset(data, along = 'metric', indices = match(metrics, metrics.loaded))
  attributes(input_data)$metrics <- metrics
      
  # Transform data for scorecards by forecast month (types 2 & 4)
  transformed_data <- SC_transform_data(data = input_data,
                                        sdate_dim = 'sdate',
                                        ftime_dim = 'time')

  ## Loop over system and reference for each scorecard plot
  for (sys in 1:dim(input_data)['system']){
    for (ref in 1:dim(input_data)['reference']) {
      
      ## TO DO: Apply check to each scorecard function
      ## check dimension 'metric' exists:
      if (!("metric" %in% names(dim(input_data)))) {
        dim(input_data) <- c(metric = 1, dim(input_data))
      }
        
      #### Scorecard_type 1 ####
      ## (no transformation or reorder)
      data_sc_1 <- input_data[sys, ref,,,,]
      SC_plot_simple_by_region(data = data_sc_1,
                               system = system[sys],
                               reference = reference[ref],
                               var = var,
                               start.year = start.year,
                               end.year = end.year,
                               start.months = start.months,
                               forecast.months = forecast.months,
                               region.names = region.names,
                               metrics = metrics,
                               cross.validation = cross.validation,
                               metric.aggregation = metric.aggregation,
                               scorecard.type = 1,
                               fileout =  fileout)

      
      #### Scorecard_type 2 ####
      ## (reorder only)
      ## Scorecard type 2 is same as type 1 for only one region, therefore is
      ## only plotted if more that one region is requested
      if(dim(input_data)['region'] > 1) {
        new_order <- c('metric', 'region', 'sdate', 'time')
        data_sc_2 <- Reorder(input_data[sys, ref,,,,], new_order)
        SC_plot_simple_by_forecast_month(data = data_sc_2,
                                         system = system[sys],
                                         reference = reference[ref],
                                         var = var,
                                         start.year = start.year,
                                         end.year = end.year,
                                         start.months = start.months,
                                         forecast.months = forecast.months,
                                         region.names = region.names,
                                         metrics = metrics,
                                         cross.validation = cross.validation,
                                         metric.aggregation = metric.aggregation,
                                         scorecard.type = 2,
                                         fileout =  fileout)
      } ## close if
      
      #### Scorecard_type 3 ####
      ## (transformation only)
      data_sc_3 <- transformed_data[sys, ref,,,,]
      SC_plot_simple_by_region(data = data_sc_3,
                               system = system[sys],
                               reference = reference[ref],
                               var = var,
                               start.year = start.year,
                               end.year = end.year,
                               start.months = start.months,
                               forecast.months = forecast.months,
                               region.names = region.names,
                               metrics = metrics,
                               cross.validation = cross.validation,
                               metric.aggregation = metric.aggregation,
                               scorecard.type = 3,
                               fileout =  fileout)


      #### Scorecard_type 4 ####
      ## (transformation and reorder)
      ## Scorecard type 4 is same as type 3 for only one region, therefore is
      ## only plotted if more that one region is requested
      if(dim(input_data)['region'] > 1) {
        new_order <- c('metric', 'region', 'sdate', 'time')
        data_sc_4 <- Reorder(transformed_data[sys, ref,,,,], new_order)
        SC_plot_simple_by_forecast_month(data = data_sc_4,
                                         system = system[sys],
                                         reference = reference[ref],
                                         var = var,
                                         start.year = start.year,
                                         end.year = end.year,
                                         start.months = start.months,
                                         forecast.months = forecast.months,
                                         region.names = region.names,
                                         metrics = metrics,
                                         cross.validation = cross.validation,
                                         metric.aggregation = metric.aggregation,
                                         scorecard.type = 4,
                                         fileout =  fileout)
      } ## close if
      
    } ## close loop on ref
  } ## close loop on sys
  
  print("All scorecard plots created")
        
} ## close function      
    
    

 