#' Scorecards function create simple scorecards by region (types 1 & 3)
#' 
#'@description function to create a scorecard for a single system and reference 
#'  combination, showing data by region and forecast month
#'  
#'@param data is an array of spatially aggregated metrics data, containing the
#'   following dimensions; metric, region, sdate, ftime.
#'@param system a character string defining the system following the
#'   archive.yml format from verification suite 
#'@param reference a character string defining the name of the reference 
#'  following the archive.yml format from verification suite 
#'@param var a character string following the format from 
#'  variable-dictionary.yml from verification suite 
#'@param start.year a numeric indicating the start year of the reference period
#'@param end.year a numeric indicating the end year of the reference period
#'@param start.date a vector of character strings indicating the start months
#'@param forecast.month a vector of numeric indicating the forecast months
#'@param region.names a vector of character strings containing names of the 
#'  regions corresponding to the input data
#'@param metrics a vector of character strings containing the metrics,  
#'  NOTE: must be in same order as in data.
#'@param scorecard.type a numeric indicating the type of scorecard to plot
#'@param fileout a path of the location to save the scorecard plots
#'  
#'@example 
#'scorecard_plot <- SC_plot_simple_by_region(data = aggregated_metrics[1,1,,,,],
#'                                           system = 'ecmwfs5' ,
#'                                           reference = 'era5',
#'                                           var = 'tas',
#'                                           start.year = 1993,
#'                                           end.year = 2016,
#'                                           start.months = 1:12
#'                                           forecast.months = 1:6,
#'                                           region.names = c('Tropics', 'Extra-tropical NH', 'Extra-tropical SH')
#'                                           metrics = c('mean_bias', 'enscorr', 'rpss','crpss', 'enssprerr'),
#'                                           scorecard.type = 1,
#'                                           fileout = '/esarchive/scratch/nmilders/scorecards_data/saved_results'
#'                                           )

SC_plot_simple_by_region <- function(data,
                                     system,
                                     reference,
                                     var,
                                     start.year,
                                     end.year,
                                     start.months,
                                     forecast.months,
                                     region.names,
                                     metrics,
                                     cross.validation,
                                     metric.aggregation,
                                     scorecard.type,
                                     fileout){

# ## For testing:
# data <- input_data[1,1,,,,] #aggregated_metrics[1,1,,,,]
# system <- c('ecmwfs5')
# reference <-'era5'
# var <- 'tas'
# start.year <- 1993
# end.year <- 2016
# start.months <- sprintf("%02d", 1:12)
# forecast.months <- 1:6
# region.names <- c('Tropics', 'Extra-tropical NH', 'Extra-tropical SH')
# metrics <- c('mean_bias', 'enscorr', 'rpss','rpss_score_aggr', 'enssprerr')
# scorecard.type <- 1
# cross.validation <- 'true'
# metric.aggregation <- 'skill'
# fileout <- '/esarchive/scratch/nmilders/saved_scorecards/testing'

  ## CHECKS
  
  # ## check dimension 'metric' exists:
  # if (!("metric" %in% names(dim(data)))) {
  #   dim(data) <- c(metric = 1, dim(data))
  # }
  # 
  # ## Make sure input_data is in correct order for using in functions:
  # dim_order <- c('metric','time','sdate','region')
  # data <- Reorder(data, dim_order)
  
  
  ## Load configuration files
  sys_dict <- read_yaml("./conf/output_dictionaries/scorecards.yml")
  var_dict <- read_yaml("./conf/variable-dictionary.yml")$vars
  
  ## Load functions
  source("./modules/Scorecards/SC_utils.R")
  source("./modules/Scorecards/SC_breaks.R")
  source("./modules/Scorecards/SC_table_colors.R")
  source("./modules/Scorecards/SC_legend.R")
  source("./modules/Scorecards/SC_save_scorecard.R")
  
  ## get dimensions of inputs
  n.start.months <- as.numeric(dim(data)["sdate"]) ## TO DO: create sdate_dim
  n.ftime.months <- as.numeric(dim(data)["time"]) ## TO DO: create ftime_dim
  n.metrics <- length(metrics) 
  n.regions <- length(region.names) 

  ## Define table size
  n.rows <- n.regions * n.ftime.months 
  n.columns <- 2 + (n.metrics * n.start.months)

  ## Get display names from configuration files
  var.name <- var_dict[[var]]$long_name
  var.units <- var_dict[[var]]$units
  system.name <- sys_dict$System[[which(sys_dict$System %like% system)]]$display_name
  reference.name <- sys_dict$Reference[[reference]]$display_name

  ## Get metric long names
  metric.names.list <- .met_names(metrics, var.units)
  
  ## format the metric names as character instead of list
  for(met in metrics){
    if(met == metrics[1]){
      metric.names <- metric.names.list[[met]] 
    } else {
      metric.names <- c(metric.names, metric.names.list[[met]])
    }
  }
  
  breaks_bias <- NULL
  
  ## Find position of mean bias metric to calculate breaks
  if ('mean_bias' %in% metrics){
    stopifnot(identical(names(dim(data)),c('metric','time','sdate','region')))
    pos_bias <- which(metrics == 'mean_bias')
    if(var == 'psl'){
      data[pos_bias,,,] <- data[pos_bias,,,]/100
    }
    breaks_bias <- SC_breaks(Subset(data, along = 'metric',
                                    indices = pos_bias))
  }
  
  ## Region layout for plotting
  region.names.figure <- rep("", n.regions * n.ftime.months)
  for(reg in 1:n.regions) region.names.figure[floor(n.ftime.months/2) + (reg-1) * n.ftime.months + 1] = region.names[reg]
  
  ## Define scorecard table titles
  column.names <- c("Region", "Forecast month", rep(c(month.abb[as.numeric(start.months)]), n.metrics))
  
  if(scorecard.type == 1){
    column.header <- "Start date"
  }
  if(scorecard.type == 3){
    column.header <- "Target month"
  }
  
  ## Round data
  data <- round(data,2)
  
  ## Define data inside the scorecards table
  ## NEED TO FIX: allow different or of metrics than in data
  for(reg in 1:n.regions){
      table_metric <- data.frame(table_column_2 = as.character(forecast.months))
      for(met in 1:n.metrics){
        table_metric <- data.frame(table_metric, data[met,,,reg])   
      } ## close loop on met
      if(reg == 1){
        table_reg <- table_metric
      } else {
        table_reg <- rbind(table_reg, table_metric)
      }
  } ## close loop on reg
  
  ## All data for plotting in table
  table <- data.frame(table_column_1 = region.names.figure, table_reg)

  ## Define breaks for each metric based of metric position:
  metric.breaks <- .met_breaks(metrics, breaks_bias)
  
  ## Set table background colors
  palette <- c('#2D004B', '#542789', '#8073AC', '#B2ABD2', '#D8DAEB', '#FEE0B6', '#FDB863', '#E08214', '#B35806', '#7F3B08')
  colorunder <- "#04040E"
  colorsup <- "#730C04"

  metric.col <- array(colorunder, c(n.ftime.months * n.regions, n.columns-2)) 
  metric.text.col <- array("#2A2A2A", c(n.ftime.months * n.regions, n.columns-2)) 
  # metric.text.bold <- array(TRUE, c(n.ftime.months * n.regions, n.columns-2)) ## Setting all values to bold
  
  ## Define colors to show in table
  table_colors  <- SC_table_colors(table,
                                    n.rows,
                                    n.columns,
                                    metrics,
                                    metric.breaks,
                                    n.start.months,
                                    palette,
                                    colorunder,
                                    colorsup,
                                    metric.col,
                                    metric.text.col)
  
  metric.col <- table_colors$metric.col
  metric.text.col <- table_colors$metric.text.col
  # metric.text.bold <- table_colors$metric.text.bold
  
  options(stringsAsFactors = FALSE) 
  table.title <- data.frame(c1=paste0(var.name, " of ", system.name, "  (cross-validation = ", tolower(toString(cross.validation)), ",  ", metric.aggregation, " aggregation)"), c2 = n.columns)
  header.names <- as.data.frame(data.frame(c1=c("", metric.names), c2=c(2, rep(n.start.months, n.metrics))))
  header.names2 <- as.data.frame(data.frame(c1=c("", paste0(rep(column.header, n.metrics))), c2=c(2, rep(n.start.months, n.metrics))))
  ref.label <- data.frame(c1=paste0("(Ref: ", reference.name, " ", start.year, "-", end.year, ")"), c2 = n.columns)
  title.space <- data.frame(c1 = "\n", c2 = n.columns)
  
  ## Create HTML table
  table.html.part <- list()
  table.html.part[[1]] <- kbl(table, escape = F, col.names = column.names, align = rep("c", n.columns)) %>%
    kable_paper("hover", full_width = F) %>%
    add_header_above(header = header.names2, font = 14)%>%
    add_header_above(header = ref.label, align = "left", font_size = 14) %>% 
    add_header_above(header = header.names, font_size = 20) %>%
    add_header_above(header = title.space, font_size = 14) %>%
    add_header_above(header = table.title, font_size = 20, align = "left")
  
  for(i in 1:n.metrics){ 
     for(stm in 1:n.start.months){ 
      my.background <- metric.col[(i - 1) * n.start.months + stm]
      my.text.color <- metric.text.col[(i - 1) * n.start.months + stm ]
      #my.bold <- metric.text.bold[(i - 1) * n.start.months + stm]

      table.html.part[[(i - 1) * n.start.months + stm + 1]] <-
        column_spec(table.html.part[[(i - 1) * n.start.months + stm]], 2 + n.start.months * (i - 1) + stm, 
                    background = my.background[1:n.rows,], 
                    color = my.text.color[1:n.rows,], 
                    bold = T) ## strsplit(toString(bold), ', ')[[1]]
      } ## close loop on stm
  } ## close loop on i
  
  n.last.list <- n.metrics * n.start.months + 1
  
  column.borders <- NULL
  
  for(i in 1:n.metrics){
    column.spacing <- (n.start.months * i) + 2
    column.borders <- c(column.borders, column.spacing)
  }

  table.html <- column_spec(table.html.part[[n.last.list]], 1, bold = TRUE, width_min = "4cm", width_max = "4cm") %>%
    column_spec(1, bold = TRUE, width_min = "4cm", width_max = "4cm") %>% 
    column_spec(2, bold = TRUE, width_max = "1cm") %>% 
    column_spec(3:n.columns, width_max = "1.1cm", width_min = "1.1cm") %>%
    column_spec(c(1, 2, column.borders), border_right = "2px solid black") %>% 
    column_spec(1, border_left = "2px solid black") %>%
    column_spec(n.columns, border_right = "2px solid black") %>%
    row_spec(seq(from = 0, to = n.ftime.months * n.regions, by = n.ftime.months), extra_css = "border-bottom: 2px solid black", hline_after = TRUE) #1px solid")

 currentworkingdir <- getwd()
  
 ## Create color bar legend
 scorecard_legend <- SC_legend(table.html,
                                metrics,
                                metric.breaks,
                                palette,
                                colorunder,
                                colorsup,
                                fileout)
          
  ## Save scorecard png 
  dirout <- save_scorecard_single(system, 
                                  reference, 
                                  var, 
                                  start.year, 
                                  end.year, 
                                  scorecard.type, 
                                  cross.validation,
                                  metric.aggregation)
   
  ## Move scorecard image to desired output folder
  system(paste0("mv ", './scorecard.png', paste0(" ", dirout))) 
  print(paste0("Scorecard type ", scorecard.type, " figure saved (sys: ", system.name, " & ref: ", reference.name, ")" ))
  
  setwd(currentworkingdir)

}## close function

